"""Standard tests."""

from __future__ import annotations

from typing import TYPE_CHECKING

import pytest
from bs4 import BeautifulSoup

if TYPE_CHECKING:
    from io import StringIO

    from sphinx.testing.util import SphinxTestApp


@pytest.mark.sphinx("html", confoverrides={"html_theme": "bulma-basic"})
def test__added_attr_in_toc(app: SphinxTestApp, status: StringIO, warning: StringIO):
    app.build()
    soup = BeautifulSoup((app.outdir / "index.html").read_text(), "html.parser")
    toc_h3 = soup.find(lambda tag: tag.name == "h3" and tag.text == "Table of Contents")  # type: ignore[union-attr]
    assert "menu-list" in toc_h3.parent.find("ul").attrs["class"]  # type: ignore[union-attr]


@pytest.mark.sphinx(
    "html",
    confoverrides={
        "html_theme": "bulma-basic",
        "html_sidebars": {
            "**": ["sidebar/globaltoc.html"],
        },
    },
    testroot="toc",
)
def test__added_attr_in_globaltoc(
    app: SphinxTestApp, status: StringIO, warning: StringIO
):
    app.build()
    soup = BeautifulSoup((app.outdir / "index.html").read_text(), "html.parser")
    toc_h3 = soup.find(lambda tag: tag.name == "h3" and tag.text == "Table of Contents")  # type: ignore[union-attr]
    assert "menu-list" in toc_h3.parent.find("ul").attrs["class"]  # type: ignore[union-attr]



"""Translation of menu items.

:ref: https://bulma.io/documentation/components/menu/
"""

from __future__ import annotations

from bs4 import BeautifulSoup, Tag


def append_style(html: str, tag: str = "ul") -> str:
    """Append class attribute to render as Bulma's menu list.

    :param html: HTML string to be modified.
    :param tag: Tag name to be searched for.
    :return: Modified HTML string.
    """
    soup = BeautifulSoup(html, "html.parser")
    elm = soup.find(tag)
    if isinstance(elm, Tag):
        elm.attrs.setdefault("class", [])
        if isinstance(elm.attrs["class"], str):
            elm.attrs["class"] = [elm.attrs["class"]]
        elm.attrs["class"].append("menu-list")
    return str(soup)



"""Entrypoint of theme."""

from pathlib import Path
from typing import Union

from bs4 import BeautifulSoup, Tag
from docutils import nodes
from sphinx.application import Sphinx

from ... import __version__
from ...components import menu
from ...components.navbar import register_root_toctree_dict

here = Path(__file__).parent


def append_styling_filters(app: Sphinx):
    """Append custom filters to update content for enabling Bulma styles."""
    app.builder.templates.environment.filters["set_menu_list"] = menu.append_style


def setup(app: Sphinx):  # noqa: D103
    app.add_html_theme("bulma-basic", str(here))
    app.connect("builder-inited", append_styling_filters)
    app.connect("html-page-context", register_root_toctree_dict)
    app.setup_extension("atsphinx.bulma")
    return {
        "version": __version__,
        "env_version": 1,
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }



def test_menu_list():
    from atsphinx.bulma.components import menu

    src = "<ul><li>Item 1</li><li>Item 2</li></ul>"
    dest = menu.append_style(src, "ul")
    assert dest == '<ul class="menu-list"><li>Item 1</li><li>Item 2</li></ul>'
